namespace Fractions

sequence pureFractions = {{0,1}, {0,1}, {0,1}}
sequence mixedFractions = {{0,0,1}, {0,0,1}, {0,0,1}}

enum first = 1, second, answer,
	pureNumerater = 1, pureDenominator,
	mixedIntPart = 1, mixedNumeraterPart, mixedDenominatorPart
	
function greatestCommonDivisor (sequence fraction)
	-- Euclid's algorythm
	integer a = abs(fraction [1]), b = abs(fraction [2])
	integer c = remainder (a,b)
	while c do
		a = b
		b = c
		c = remainder (a,b)
	end while
	return b
end function

public function leastCommonMustiple (integer a, integer b)
	-- Not used, just here for the sake of completeness
	return (a * b) / greatestCommonDivisor ({a, b})
end function

procedure reduceIt (integer fract)
	if fract < 1 or fract > 3 then return end if
	sequence fraction = pureFractions [fract]
	integer c = greatestCommonDivisor (fraction)
	fraction [pureNumerater] /= c
	fraction [pureDenominator] /= c
	pureFractions [fract] = fraction
end procedure

procedure makePureFraction (integer fract)
	pureFractions [fract] [pureNumerater] = mixedFractions [fract] [mixedIntPart] * 
		mixedFractions [fract] [mixedDenominatorPart] + mixedFractions [fract] [mixedNumeraterPart]
	pureFractions [fract] [pureDenominator] = mixedFractions [fract] [mixedDenominatorPart]
	reduceIt (fract)
end procedure

procedure makeMixedFraction (integer fract)
	reduceIt (fract)
	integer cSign
	sequence fraction = pureFractions [fract]
	integer top = fraction[pureNumerater]
	integer bottom = fraction [pureDenominator]
	cSign = 0
	if (top < 0 and bottom > 0) or (top > 0 and bottom < 0) then cSign = 1 end if
	top = abs (top)
	bottom = abs (bottom)
	integer intPart = floor (top / bottom)
	integer topPart = remainder (top, bottom)
	if cSign then
		topPart = -topPart
		intPart = -intPart
	end if
	fraction = {intPart, topPart, bottom}
	mixedFractions [fract] = fraction
end procedure

public procedure reduce (integer fract)
	makeMixedFraction ( fract)
end procedure
	
-- Pure fractions get and set
public function set_pureNumerater (integer fract, integer fr)
	if fract < 1 or fract > 3 then return 0 end if
	pureFractions [fract] [pureNumerater] = fr
	makeMixedFraction (fract)
	return 1
end function

public function get_pureNumerater (integer fract)
	if fract < 1 or fract > 3 then return 0 end if
	return pureFractions [fract] [pureNumerater]
end function

public function set_pureDenominator (integer fract, integer fr)
	if fract < 1 or fract > 3 then return 0 end if
	if fr = 0 then return 0 end if
	pureFractions [fract] [pureDenominator] = fr
	makeMixedFraction (fract)
	return 1
end function

public function get_pureDenominator (integer fract)
	if fract < 1 or fract > 3 then return 0 end if
	return pureFractions [fract] [pureDenominator]
end function

public function set_pureFraction (integer fract, sequence fr)
	if fract < 1 or fract > 3 then return 0 end if
	if length (fr) != 2 then return 0 end if
	if fr [pureDenominator] = 0 then return 0 end if
	pureFractions [fract] = fr

	makeMixedFraction (fract)
	return 1
end function

public function get_pureFraction (integer fract)
	if fract < 1 or fract > 3 then return 0 end if
	return pureFractions [fract]
end function

-- Mixed fractions get and set
public function set_mixedIntPart (integer fract, integer fr)
	if fract < 1 or fract > 3 then return 0 end if
	mixedFractions [fract] [mixedIntPart] = fr
	makePureFraction (fract)
	return 1
end function

public function get_mixedIntPart (integer fract)
	if fract < 1 or fract > 3 then return 0 end if
	return mixedFractions [fract] [mixedIntPart]
end function

public function set_mixedNumeraterPart (integer fract, integer fr)
	if fract < 1 or fract > 3 then return 0 end if
	mixedFractions [fract] [mixedNumeraterPart] = fr
	makePureFraction (fract)
	return 1
end function

public function get_mixedNumeraterPart (integer fract)
	if fract < 1 or fract > 3 then return 0 end if
	return mixedFractions [fract] [mixedNumeraterPart]
end function

public function set_mixedDenominatorPart (integer fract, integer fr)
	if fract < 1 or fract > 3 then return 0 end if
	if fr = 0 then return 0 end if
	mixedFractions [fract] [mixedDenominatorPart] = fr
	makePureFraction (fract)
	return 1
end function

public function get_mixedDenominatorPart (integer fract)
	if fract < 1 or fract > 3 then return 0 end if
	return mixedFractions [fract] [mixedDenominatorPart]
end function

public function set_mixedFraction (integer fract, sequence fr)
	if fract < 1 or fract > 3 then return 0 end if
	if length (fr) != 3 then return 0 end if
	if fr [mixedDenominatorPart] = 0 then return 0 end if
	mixedFractions [fract] [mixedIntPart] = fr [mixedIntPart]
	mixedFractions [fract] [mixedNumeraterPart] = fr [mixedNumeraterPart]
	mixedFractions [fract] [mixedDenominatorPart] = fr [mixedDenominatorPart]
	makePureFraction (fract)
	return 1
end function

public function get_mixedFraction (integer fract)
	if fract < 1 or fract > 3 then return 0 end if
	return mixedFractions [fract] 
end  function

-- Arithmetic operations on pure fractions
public procedure add ()
	pureFractions [answer] [pureNumerater] = pureFractions [first] [pureNumerater] *
		pureFractions [second] [pureDenominator] + 
		pureFractions [second] [pureNumerater] *
		pureFractions [first] [pureDenominator]
	pureFractions [answer] [pureDenominator] = pureFractions [first] [pureDenominator] *
		pureFractions [second] [pureDenominator]
	makeMixedFraction (answer)
end procedure

public procedure subtract ()
	pureFractions [answer] [pureNumerater] = pureFractions [first] [pureNumerater] *
		pureFractions [second] [pureDenominator] - 
		pureFractions [second] [pureNumerater] *
		pureFractions [first] [pureDenominator]
	pureFractions [answer] [pureDenominator] = pureFractions [first] [pureDenominator] *
		pureFractions [second] [pureDenominator]
	makeMixedFraction (answer)
end procedure

public procedure mult ()
	pureFractions [answer] [pureNumerater] = pureFractions [first] [pureNumerater] *
		pureFractions [second] [pureNumerater]
	pureFractions [answer] [pureDenominator] = pureFractions [first] [pureDenominator] *
		pureFractions [second] [pureDenominator]
	makeMixedFraction (answer)
end procedure

public procedure multiply ()
	mult ()
end procedure

public procedure divide ()
	pureFractions [answer] [pureNumerater] = pureFractions [first] [pureNumerater] *
		pureFractions [second] [pureDenominator]
	pureFractions [answer] [pureDenominator] = pureFractions [first] [pureDenominator] *
		pureFractions [second] [pureNumerater]
	makeMixedFraction (answer)
end procedure
	
	
	
	
	
